import pya

# Instances of devices to be labeled are named "Chip"
# and are children of the "Wafer" cell
ly = pya.CellView().active().layout()
wafer = ly.cell("Wafer")

# Gather coordinates of each chip instance   
chip_coordinates = []
for inst in wafer.each_inst():
    if inst.cell.name == "Chip":
        for array_trans in inst.cell_inst.each_trans():
                chip_coordinates.append(array_trans.disp)
# Sort chip coordinates so they are ordered left-to-right and top-to-bottom (like a page in a book)
chip_coordinates = sorted(chip_coordinates, key= lambda x: (x.y,-1*x.x), reverse=True)

# Set relative position of the Serial Number on chip
sn_x = -5
sn_y = 4
# Text magnification (i.e. relative font size)
mag = 2.0
# layer where to put the serial number
layer = pya.LayerInfo.new(1, 0)
# make cell to contain the serial numbers
serial_numbers = ly.create_cell("SerialNumbers")
sn_format = "A%03d"

#Apply serial numbers for each Chipset
sn = 1
for pt in chip_coordinates:
    X = pt.x + rel.x  + sn_x / ly.dbu
    Y = pt.y + rel.y  + sn_y / ly.dbu
    trans = pya.Trans.new(pya.Vector.new(X,Y))
    text =  sn_format % (sn)
    sn_param = {"layer" : layer, "text" :text, "mag" : mag}
    sn_cell = ly.create_cell("TEXT", "Basic", sn_param)
    serial_numbers.insert(pya.CellInstArray.new(sn_cell.cell_index(), trans))
    sn = sn+1


#Add the serial number text cell instance to the wafer cell
trans = pya.Trans.new(pya.Vector.new(0,0))
wafer.insert(pya.CellInstArray.new(serial_numbers.cell_index(), trans))